@@ -28,6 +28,7 @@ THE SOFTWARE.
 """
 
 import builtins
+import contextlib
 import logging
 import operator
 import re
@@ -55,6 +56,11 @@ from typing import (
 
 from typing_extensions import Self, dataclass_transform
 
+from pytools.version import VERSION_TEXT
+
+
+__version__ = VERSION_TEXT
+
 
 # These are deprecated and will go away in 2022.
 all = builtins.all
@@ -416,10 +422,8 @@ class RecordWithoutPickling:
     def get_copy_kwargs(self, **kwargs):
         for f in self.__class__.fields:
             if f not in kwargs:
-                try:
+                with contextlib.suppress(AttributeError):
                     kwargs[f] = getattr(self, f)
-                except AttributeError:
-                    pass
         return kwargs
 
     def copy(self, **kwargs):
@@ -610,10 +614,7 @@ def is_single_valued(
     except StopIteration:
         raise ValueError("empty iterable passed to 'single_valued()'") from None
 
-    for other_item in it:
-        if not equality_pred(other_item, first_item):
-            return False
-    return True
+    return all(equality_pred(other_item, first_item) for other_item in it)
 
 
 all_equal = is_single_valued
@@ -637,12 +638,7 @@ def single_valued(
     except StopIteration:
         raise ValueError("empty iterable passed to 'single_valued()'") from None
 
-    def others_same():
-        for other_item in it:
-            if not equality_pred(other_item, first_item):
-                return False
-        return True
-    assert others_same()
+    assert all(equality_pred(other_item, first_item) for other_item in it)
 
     return first_item
 
@@ -720,7 +716,7 @@ def memoize(*args: F, **kwargs: Any) -> F:
             return wrapper
 
     if not args:
-        return _decorator  # type: ignore
+        return _decorator  # type: ignore[return-value]
     if callable(args[0]) and len(args) == 1:
         return _decorator(args[0])
     raise TypeError(
@@ -749,10 +745,7 @@ def memoize_on_first_arg(
                 )
 
     def wrapper(obj: T, *args: P.args, **kwargs: P.kwargs) -> R:
-        if kwargs:
-            key = (_HasKwargs, frozenset(kwargs.items()), *args)
-        else:
-            key = args
+        key = (_HasKwargs, frozenset(kwargs.items()), *args) if kwargs else args
 
         assert cache_dict_name is not None
         try:
@@ -1658,6 +1651,41 @@ class Table:
 
         return "\n".join(lines)
 
+    def str_with_maxlen(self, maxlen: int) -> str:
+        """
+        Returns a string representation of the table, limiting lines to
+        *maxlen*. Only makes sense with 'left' alignment in the last column.
+
+        .. doctest ::
+
+            >>> tbl = Table(alignments=['l', 'l'])
+            >>> tbl.add_row([0, 'normal column'])
+            >>> tbl.add_row([1, 'very very very very long column'])
+            >>> tbl.add_row([10, '20'])
+            >>> print(tbl.str_with_maxlen(15))
+            0  | normal ...
+            ---+-----------
+            1  | very v ...
+            10 | 20
+        """
+        if self._get_alignments()[-1] != "l":
+            from warnings import warn
+            warn("str_with_maxlen() only makes sense with 'left' "
+                 "alignment in the last column", stacklevel=2)
+
+        lines = str(self).split("\n")
+        lines = [line.rstrip() for line in lines]
+
+        for i, line in enumerate(lines):
+            if len(line) > maxlen:
+                if i == 1:
+                    # Header separator line
+                    lines[i] = line[:maxlen]
+                else:
+                    lines[i] = line[:maxlen-4] + " ..."
+
+        return "\n".join(lines)
+
     def github_markdown(self) -> str:
         r"""Returns a string representation of the table formatted as
         `GitHub-Flavored Markdown.
@@ -1920,89 +1948,6 @@ def word_wrap(text, width, wrap_using="\n"):
 # }}}
 
 
-# {{{ command line interfaces
-
-def _exec_arg(arg, execenv):
-    import os
-    if os.access(arg, os.F_OK):
-        exec(compile(open(arg), arg, "exec"), execenv)
-    else:
-        exec(compile(arg, "<command line>", "exec"), execenv)
-
-
-class CPyUserInterface:
-    class Parameters(Record):
-        pass
-
-    def __init__(self, variables, constants=None, doc=None):
-        if constants is None:
-            constants = {}
-        if doc is None:
-            doc = {}
-        self.variables = variables
-        self.constants = constants
-        self.doc = doc
-
-    def show_usage(self, progname):
-        print(f"usage: {progname} <FILE-OR-STATEMENTS>")
-        print()
-        print("FILE-OR-STATEMENTS may either be Python statements of the form")
-        print("'variable1 = value1; variable2 = value2' or the name of a file")
-        print("containing such statements. Any valid Python code may be used")
-        print("on the command line or in a command file. If new variables are")
-        print("used, they must start with 'user_' or just '_'.")
-        print()
-        print("The following variables are recognized:")
-        for v in sorted(self.variables):
-            print(f"  {v} = {self.variables[v]}")
-            if v in self.doc:
-                print(f"    {self.doc[v]}")
-
-        print()
-        print("The following constants are supplied:")
-        for c in sorted(self.constants):
-            print(f"  {c} = {self.constants[c]}")
-            if c in self.doc:
-                print(f"    {self.doc[c]}")
-
-    def gather(self, argv=None):
-        if argv is None:
-            argv = sys.argv
-
-        if len(argv) == 1 or (
-                ("-h" in argv)
-                or ("help" in argv)
-                or ("-help" in argv)
-                or ("--help" in argv)):
-            self.show_usage(argv[0])
-            sys.exit(2)
-
-        execenv = self.variables.copy()
-        execenv.update(self.constants)
-
-        for arg in argv[1:]:
-            _exec_arg(arg, execenv)
-
-        # check if the user set invalid keys
-        for added_key in (
-                set(execenv.keys())
-                - set(self.variables.keys())
-                - set(self.constants.keys())):
-            if not (added_key.startswith("user_") or added_key.startswith("_")):
-                raise ValueError(
-                        f"invalid setup key: '{added_key}' "
-                        "(user variables must start with 'user_' or '_')")
-
-        result = self.Parameters({key: execenv[key] for key in self.variables})
-        self.validate(result)
-        return result
-
-    def validate(self, setup):
-        pass
-
-# }}}
-
-
 # {{{ debugging
 
 class StderrToStdout:
@@ -2088,9 +2033,8 @@ def invoke_editor(s, filename="edit.txt", descr="the file"):
     from os.path import join
     full_name = join(tempdir, filename)
 
-    outf = open(full_name, "w")
-    outf.write(str(s))
-    outf.close()
+    with open(full_name, "w") as outf:
+        outf.write(str(s))
 
     import os
     if "EDITOR" in os.environ:
@@ -2102,9 +2046,8 @@ def invoke_editor(s, filename="edit.txt", descr="the file"):
                 "dropped directly into an editor next time.)")
         input(f"Edit {descr} at {full_name} now, then hit [Enter]:")
 
-    inf = open(full_name)
-    result = inf.read()
-    inf.close()
+    with open(full_name) as inf:
+        result = inf.read()
 
     return result
 
@@ -2122,7 +2065,8 @@ class ProgressBar:
     .. automethod:: __enter__
     .. automethod:: __exit__
     """
-    def __init__(self, descr, total, initial=0, length=40):
+    def __init__(self, descr: str, total: int, initial: int = 0,
+                 length: int = 40) -> None:
         import time
         self.description = descr
         self.total = total
@@ -2135,9 +2079,9 @@ class ProgressBar:
         self.speed_meas_start_time = self.start_time
         self.speed_meas_start_done = initial
 
-        self.time_per_step = None
+        self.time_per_step: float | None = None
 
-    def draw(self):
+    def draw(self) -> None:
         import time
 
         now = time.time()
@@ -2170,21 +2114,21 @@ class ProgressBar:
             self.last_squares = squares
             self.last_update_time = now
 
-    def progress(self, steps=1):
+    def progress(self, steps: int = 1) -> None:
         self.set_progress(self.done + steps)
 
-    def set_progress(self, done):
+    def set_progress(self, done: int) -> None:
         self.done = done
         self.draw()
 
-    def finished(self):
+    def finished(self) -> None:
         self.set_progress(self.total)
         sys.stderr.write("\n")
 
-    def __enter__(self):
+    def __enter__(self) -> None:
         self.draw()
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self.finished()
 
 # }}}
@@ -2628,16 +2572,14 @@ class ProcessLogger:
             use_late_start_logging = False
 
         if use_late_start_logging:
-            try:
+            # https://github.com/firedrakeproject/firedrake/issues/1422
+            #
+            # Starting a thread may fail in various environments, e.g. MPI.
+            # Since the late-start logging is an optional 'quality-of-life'
+            # feature for interactive use, tolerate failures of it without
+            # warning.
+            with contextlib.suppress(RuntimeError):
                 self.late_start_log_thread.start()
-            except RuntimeError:
-                # https://github.com/firedrakeproject/firedrake/issues/1422
-                #
-                # Starting a thread may fail in various environments, e.g. MPI.
-                # Since the late-start logging is an optional 'quality-of-life'
-                # feature for interactive use, tolerate failures of it without
-                # warning.
-                pass
 
         self.timer = ProcessTimer()
 
