@@ -1,8 +1,13 @@
-from typing import IO, Any, Callable, Iterator, List, Optional, Sequence, Tuple
+from __future__ import annotations
+
+from typing import IO, TYPE_CHECKING, Any
 
 from pytools import Record
 
 
+if TYPE_CHECKING:
+    from collections.abc import Callable, Iterator, Sequence
+
 __doc__ = """
 An in-memory relational database table
 ======================================
@@ -25,7 +30,7 @@ class DataTable:
     """
 
     def __init__(self, column_names: Sequence[str],
-                 column_data: Optional[List[Any]] = None) -> None:
+                 column_data: list[Any] | None = None) -> None:
         """Construct a new table, with the given C{column_names}.
 
         :arg column_names: An indexable of column name strings.
@@ -50,7 +55,7 @@ class DataTable:
     def __len__(self) -> int:
         return len(self.data)
 
-    def __iter__(self) -> Iterator[List[Any]]:
+    def __iter__(self) -> Iterator[list[Any]]:
         return self.data.__iter__()
 
     def __str__(self) -> str:
@@ -65,7 +70,7 @@ class DataTable:
 
         def format_row(row: Sequence[str]) -> str:
             return "|".join([str(cell).ljust(col_width)
-                      for cell, col_width in zip(row, col_widths)])
+                      for cell, col_width in zip(row, col_widths, strict=True)])
 
         lines = [format_row(self.column_names),
                 "+".join("-"*col_width for col_width in col_widths)] + \
@@ -80,16 +85,16 @@ class DataTable:
 
         self.insert_row(tuple(values))
 
-    def insert_row(self, values: Tuple[Any, ...]) -> None:
+    def insert_row(self, values: tuple[Any, ...]) -> None:
         assert isinstance(values, tuple)
         assert len(values) == len(self.column_names)
         self.data.append(values)
 
-    def insert_rows(self, rows: Sequence[Tuple[Any, ...]]) -> None:
+    def insert_rows(self, rows: Sequence[tuple[Any, ...]]) -> None:
         for row in rows:
             self.insert_row(row)
 
-    def filtered(self, **kwargs: Any) -> "DataTable":
+    def filtered(self, **kwargs: Any) -> DataTable:
         if not kwargs:
             return self
 
@@ -118,19 +123,19 @@ class DataTable:
         if len(filtered) > 1:
             raise RuntimeError("more than one matching entry for get()")
 
-        return Row(dict(list(zip(self.column_names, filtered.data[0]))))
+        return Row(dict(zip(self.column_names, filtered.data[0], strict=True)))
 
     def clear(self) -> None:
         del self.data[:]
 
-    def copy(self) -> "DataTable":
+    def copy(self) -> DataTable:
         """Make a copy of the instance, but leave individual rows untouched.
 
         If the rows are modified later, they will also be modified in the copy.
         """
         return DataTable(self.column_names, self.data[:])
 
-    def deep_copy(self) -> "DataTable":
+    def deep_copy(self) -> DataTable:
         """Make a copy of the instance down to the row level.
 
         The copy's rows may be modified independently from the original.
@@ -140,7 +145,7 @@ class DataTable:
     def sort(self, columns: Sequence[str], reverse: bool = False) -> None:
         col_indices = [self.column_indices[col] for col in columns]
 
-        def mykey(row: Sequence[Any]) -> Tuple[Any, ...]:
+        def mykey(row: Sequence[Any]) -> tuple[Any, ...]:
             return tuple(
                     row[col_index]
                     for col_index in col_indices)
@@ -148,7 +153,7 @@ class DataTable:
         self.data.sort(reverse=reverse, key=mykey)
 
     def aggregated(self, groupby: Sequence[str], agg_column: str,
-                   aggregate_func: Callable[[Sequence[Any]], Any]) -> "DataTable":
+                   aggregate_func: Callable[[Sequence[Any]], Any]) -> DataTable:
         gb_indices = [self.column_indices[col] for col in groupby]
         agg_index = self.column_indices[agg_column]
 
@@ -157,8 +162,8 @@ class DataTable:
         result_data = []
 
         # to pacify pyflakes:
-        last_values: Tuple[Any, ...] = ()
-        agg_values: List[Row] = []
+        last_values: tuple[Any, ...] = ()
+        agg_values: list[Row] = []
 
         for row in self.data:
             this_values = tuple(row[i] for i in gb_indices)
@@ -179,8 +184,8 @@ class DataTable:
                 [self.column_names[i] for i in gb_indices] + [agg_column],
                 result_data)
 
-    def join(self, column: str, other_column: str, other_table: "DataTable",
-             outer: bool = False) -> "DataTable":
+    def join(self, column: str, other_column: str, other_table: DataTable,
+             outer: bool = False) -> DataTable:
         """Return a table joining this and the C{other_table} on C{column}.
 
         The new table has the following columns:
@@ -192,7 +197,7 @@ class DataTable:
         by which they are joined.
         """
 
-        def without(indexable: Tuple[str, ...], idx: int) -> Tuple[str, ...]:
+        def without(indexable: tuple[str, ...], idx: int) -> tuple[str, ...]:
             return indexable[:idx] + indexable[idx+1:]
 
         this_key_idx = self.column_indices[column]
@@ -239,9 +244,8 @@ class DataTable:
                     except StopIteration:
                         this_over = True
                         break
-            else:
-                if outer:
-                    this_batch = [(None,) * len(self.column_names)]
+            elif outer:
+                this_batch = [(None,) * len(self.column_names)]
 
             if run_other and not other_over:
                 key = other_key
@@ -252,9 +256,8 @@ class DataTable:
                     except StopIteration:
                         other_over = True
                         break
-            else:
-                if outer:
-                    other_batch = [(None,) * len(other_table.column_names)]
+            elif outer:
+                other_batch = [(None,) * len(other_table.column_names)]
 
             for this_batch_row in this_batch:
                 for other_batch_row in other_batch:
@@ -266,19 +269,18 @@ class DataTable:
             if outer:
                 if this_over and other_over:
                     break
-            else:
-                if this_over or other_over:
-                    break
+            elif this_over or other_over:
+                break
 
         return DataTable(result_columns, result_data)
 
-    def restricted(self, columns: Sequence[str]) -> "DataTable":
+    def restricted(self, columns: Sequence[str]) -> DataTable:
         col_indices = [self.column_indices[col] for col in columns]
 
         return DataTable(columns,
                 [[row[i] for i in col_indices] for row in self.data])
 
-    def column_data(self, column: str) -> List[Tuple[Any, ...]]:
+    def column_data(self, column: str) -> list[tuple[Any, ...]]:
         col_index = self.column_indices[column]
         return [row[col_index] for row in self.data]
 
