@@ -35,21 +35,10 @@ import os
 import pickle
 import sqlite3
 import sys
+from collections.abc import Callable, Iterator, Mapping
 from dataclasses import fields as dc_fields, is_dataclass
 from enum import Enum
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    FrozenSet,
-    Iterator,
-    Mapping,
-    Optional,
-    Protocol,
-    Tuple,
-    TypeVar,
-    cast,
-)
+from typing import TYPE_CHECKING, Any, Protocol, TypeVar, cast
 from warnings import warn
 
 
@@ -79,6 +68,10 @@ else:
     _HAS_ATTRS = True
 
 
+# Used to speed up the comparison in update_for_numpy_scalar and for mock testing
+_IS_BIGENDIAN = sys.byteorder == "big"
+
+
 logger = logging.getLogger(__name__)
 
 # NOTE: not always available so they get hardcoded here
@@ -174,12 +167,6 @@ class KeyBuilder:
         may stop working as early as 2022.
 
         .. versionadded:: 2021.2
-
-    .. note::
-
-        Some key-building uses system byte order, so the built keys may not match
-        across different systems. It would be desirable to fix this, but this is
-        not yet done.
     """
 
     # this exists so that we can (conceivably) switch algorithms at some point
@@ -232,10 +219,12 @@ class KeyBuilder:
                             method = self.update_for_specific_dtype
 
                     # Hashing numpy scalars
-                    elif isinstance(key, np.number):
+                    elif isinstance(key, np.number | np.bool_):
                         # Non-numpy scalars are handled above in the try block.
                         method = self.update_for_numpy_scalar
 
+                    # numpy arrays are mutable so they are not handled in KeyBuilder
+
                 if method is None:
                     if issubclass(tp, Enum):
                         method = self.update_for_enum
@@ -274,55 +263,58 @@ class KeyBuilder:
 
     # {{{ updaters
 
-    @staticmethod
-    def update_for_type(key_hash: Hash, key: type) -> None:
+    # NOTE: None of these should be static or classmethods. While Python itself is
+    # perfectly OK with overriding those with 'normal' methods, type checkers
+    # understandably don't like it.
+
+    def update_for_type(self, key_hash: Hash, key: type) -> None:
         key_hash.update(
             f"{key.__module__}.{key.__qualname__}.{key.__name__}".encode())
 
-    update_for_ABCMeta = update_for_type  # noqa: N815
+    update_for_ABCMeta = update_for_type
 
-    @staticmethod
-    def update_for_int(key_hash: Hash, key: int) -> None:
+    def update_for_int(self, key_hash: Hash, key: int) -> None:
         sz = 8
+        # Note: this must match the hash for numpy integers, since
+        # np.int64(1) == 1
         while True:
             try:
-                # Must match system byte order so that numpy and this
-                # generate the same string of bytes.
-                # https://github.com/inducer/pytools/issues/259
-                key_hash.update(key.to_bytes(sz, byteorder=sys.byteorder, signed=True))
+                key_hash.update(key.to_bytes(sz, byteorder="little", signed=True))
                 return
             except OverflowError:
                 sz *= 2
 
-    @classmethod
-    def update_for_enum(cls, key_hash: Hash, key: Enum) -> None:
-        cls.update_for_str(key_hash, str(key))
+    def update_for_enum(self, key_hash: Hash, key: Enum) -> None:
+        self.update_for_str(key_hash, str(key))
 
-    @staticmethod
-    def update_for_bool(key_hash: Hash, key: bool) -> None:
+    def update_for_bool(self, key_hash: Hash, key: bool) -> None:
         key_hash.update(str(key).encode("utf8"))
 
-    @staticmethod
-    def update_for_float(key_hash: Hash, key: float) -> None:
+    def update_for_float(self, key_hash: Hash, key: float) -> None:
+        import math
+        if math.isnan(key):
+            # Also applies to np.nan, float("nan")
+            warn("Encountered a NaN while hashing. Since NaNs compare unequal "
+                 "to themselves, the resulting key can not be retrieved from a "
+                 "PersistentDict and will lead to a collision error on retrieval.",
+                 stacklevel=1)
+
         key_hash.update(key.hex().encode("utf8"))
 
-    @staticmethod
-    def update_for_complex(key_hash: Hash, key: float) -> None:
+    def update_for_complex(self, key_hash: Hash, key: float) -> None:
         key_hash.update(repr(key).encode("utf-8"))
 
-    @staticmethod
-    def update_for_str(key_hash: Hash, key: str) -> None:
+    def update_for_str(self, key_hash: Hash, key: str) -> None:
         key_hash.update(key.encode("utf8"))
 
-    @staticmethod
-    def update_for_bytes(key_hash: Hash, key: bytes) -> None:
+    def update_for_bytes(self, key_hash: Hash, key: bytes) -> None:
         key_hash.update(key)
 
-    def update_for_tuple(self, key_hash: Hash, key: Tuple[Any, ...]) -> None:
+    def update_for_tuple(self, key_hash: Hash, key: tuple[Any, ...]) -> None:
         for obj_i in key:
             self.rec(key_hash, obj_i)
 
-    def update_for_frozenset(self, key_hash: Hash, key: FrozenSet[Any]) -> None:
+    def update_for_frozenset(self, key_hash: Hash, key: frozenset[Any]) -> None:
         from pytools import unordered_hash
 
         unordered_hash(
@@ -330,34 +322,36 @@ class KeyBuilder:
             (self.rec(self.new_hash(), key_i).digest() for key_i in key),
             hash_constructor=self.new_hash)
 
-    update_for_FrozenOrderedSet = update_for_frozenset  # noqa: N815
+    update_for_FrozenOrderedSet = update_for_frozenset
 
-    @staticmethod
-    def update_for_NoneType(key_hash: Hash, key: None) -> None:  # noqa: N802
+    def update_for_NoneType(self, key_hash: Hash, key: None) -> None:
         del key
         key_hash.update(b"<None>")
 
-    @staticmethod
-    def update_for_dtype(key_hash: Hash, key: Any) -> None:
+    def update_for_dtype(self, key_hash: Hash, key: Any) -> None:
         key_hash.update(key.str.encode("utf8"))
 
     # Handling numpy >= 1.20, for which
     # type(np.dtype("float32")) -> "dtype[float32]"
     # Introducing this method allows subclasses to specially handle all those
     # dtypes.
-    @staticmethod
-    def update_for_specific_dtype(key_hash: Hash, key: Any) -> None:
+    def update_for_specific_dtype(self, key_hash: Hash, key: Any) -> None:
         key_hash.update(key.str.encode("utf8"))
 
-    @staticmethod
-    def update_for_numpy_scalar(key_hash: Hash, key: Any) -> None:
+    def update_for_numpy_scalar(self, key_hash: Hash, key: Any) -> None:
         import numpy as np
+
+        # tobytes() of np.complex256 and np.float128 are non-deterministic,
+        # so we need to use their repr() instead.
         if hasattr(np, "complex256") and key.dtype == np.dtype("complex256"):
             key_hash.update(repr(complex(key)).encode("utf8"))
         elif hasattr(np, "float128") and key.dtype == np.dtype("float128"):
             key_hash.update(repr(float(key)).encode("utf8"))
         else:
-            key_hash.update(np.array(key).tobytes())
+            if _IS_BIGENDIAN:
+                key_hash.update(np.array(key).byteswap().tobytes())
+            else:
+                key_hash.update(np.array(key).tobytes())
 
     def update_for_dataclass(self, key_hash: Hash, key: Any) -> None:
         self.rec(key_hash, f"{type(key).__qualname__}.{type(key).__name__}")
@@ -383,8 +377,8 @@ class KeyBuilder:
 
     update_for_immutabledict = update_for_frozendict
     update_for_constantdict = update_for_frozendict
-    update_for_PMap = update_for_frozendict  # noqa: N815
-    update_for_Map = update_for_frozendict  # noqa: N815
+    update_for_PMap = update_for_frozendict
+    update_for_Map = update_for_frozendict
 
     # {{{ date, time, datetime, timezone
 
@@ -432,24 +426,20 @@ class KeyBuilder:
 
 class NoSuchEntryError(KeyError):
     """Raised when an entry is not found in a :class:`PersistentDict`."""
-    pass
 
 
 class NoSuchEntryCollisionError(NoSuchEntryError):
     """Raised when an entry is not found in a :class:`PersistentDict`, but it
     contains an entry with the same hash key (hash collision)."""
-    pass
 
 
 class ReadOnlyEntryError(KeyError):
     """Raised when an attempt is made to overwrite an entry in a
     :class:`WriteOncePersistentDict`."""
-    pass
 
 
 class CollisionWarning(UserWarning):
     """Warning raised when a collision is detected in a :class:`PersistentDict`."""
-    pass
 
 
 def __getattr__(name: str) -> Any:
@@ -469,10 +459,10 @@ V = TypeVar("V")
 class _PersistentDictBase(Mapping[K, V]):
     def __init__(self,
                  identifier: str,
-                 key_builder: Optional[KeyBuilder] = None,
-                 container_dir: Optional[str] = None,
+                 key_builder: KeyBuilder | None = None,
+                 container_dir: str | None = None,
                  enable_wal: bool = False,
-                 safe_sync: Optional[bool] = None) -> None:
+                 safe_sync: bool | None = None) -> None:
         self.identifier = identifier
         self.conn = None
 
@@ -580,7 +570,7 @@ class _PersistentDictBase(Mapping[K, V]):
 
         return cursor
 
-    def _exec_sql_fn(self, fn: Callable[[], T]) -> Optional[T]:
+    def _exec_sql_fn(self, fn: Callable[[], T]) -> T | None:
         n = 0
 
         with self.mutex:
@@ -605,11 +595,11 @@ class _PersistentDictBase(Mapping[K, V]):
 
     def store(self, key: K, value: V, _skip_if_present: bool = False) -> None:
         """Store (*key*, *value*) in the dictionary."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def fetch(self, key: K) -> V:
         """Return the value associated with *key* in the dictionary."""
-        raise NotImplementedError()
+        raise NotImplementedError
 
     def _make_container_dir(self) -> None:
         """Create the container directory to store the dictionary."""
@@ -643,7 +633,7 @@ class _PersistentDictBase(Mapping[K, V]):
         for row in self._exec_sql("SELECT key_value FROM dict ORDER BY rowid"):
             yield pickle.loads(row[0])[1]
 
-    def items(self) -> Iterator[Tuple[K, V]]:  # type: ignore[override]
+    def items(self) -> Iterator[tuple[K, V]]:  # type: ignore[override]
         """Return an iterator over the items in the dictionary."""
         for row in self._exec_sql("SELECT key_value FROM dict ORDER BY rowid"):
             yield pickle.loads(row[0])
@@ -679,6 +669,12 @@ class WriteOncePersistentDict(_PersistentDictBase[K, V]):
         and if they occur, almost always due to a bug in the hash key
         generation code (:class:`KeyBuilder`).
 
+    .. warning::
+
+        Since NaNs compare unequal to themselves, keys that include NaNs can
+        not be retrieved from a :class:`WriteOncePersistentDict` and will lead to a
+        :exc:`NoSuchEntryCollisionError` on retrieval.
+
     .. automethod:: __init__
     .. automethod:: __getitem__
     .. automethod:: __setitem__
@@ -689,11 +685,11 @@ class WriteOncePersistentDict(_PersistentDictBase[K, V]):
     .. automethod:: fetch
     """
     def __init__(self, identifier: str,
-                 key_builder: Optional[KeyBuilder] = None,
-                 container_dir: Optional[str] = None,
+                 key_builder: KeyBuilder | None = None,
+                 container_dir: str | None = None,
                  *,
                  enable_wal: bool = False,
-                 safe_sync: Optional[bool] = None,
+                 safe_sync: bool | None = None,
                  in_mem_cache_size: int = 256) -> None:
         """
         :arg identifier: a filename-compatible string identifying this
@@ -741,16 +737,14 @@ class WriteOncePersistentDict(_PersistentDictBase[K, V]):
                     if e.sqlite_errorcode == SQLITE_CONSTRAINT_PRIMARYKEY:
                         raise ReadOnlyEntryError("WriteOncePersistentDict, "
                                                  "tried overwriting key") from e
-                    else:
-                        raise
-                else:
-                    raise ReadOnlyEntryError("WriteOncePersistentDict, "
-                                             "tried overwriting key") from e
+                    raise
+                raise ReadOnlyEntryError("WriteOncePersistentDict, "
+                                         "tried overwriting key") from e
 
-    def _fetch_uncached(self, keyhash: str) -> Tuple[K, V]:
+    def _fetch_uncached(self, keyhash: str) -> tuple[K, V]:
         # This method is separate from fetch() to allow for LRU caching
 
-        def fetch_inner() -> Optional[Tuple[Any]]:
+        def fetch_inner() -> tuple[Any] | None:
             assert self.conn is not None
 
             # This is separate from fetch() so that the mutex covers the
@@ -794,6 +788,12 @@ class PersistentDict(_PersistentDictBase[K, V]):
         and if they occur, almost always due to a bug in the hash key
         generation code (:class:`KeyBuilder`).
 
+    .. warning::
+
+        Since NaNs compare unequal to themselves, keys that include NaNs can
+        not be retrieved from a :class:`PersistentDict` and will lead to a
+        :exc:`NoSuchEntryCollisionError` on retrieval.
+
     .. automethod:: __init__
     .. automethod:: __getitem__
     .. automethod:: __setitem__
@@ -806,11 +806,11 @@ class PersistentDict(_PersistentDictBase[K, V]):
     """
     def __init__(self,
                  identifier: str,
-                 key_builder: Optional[KeyBuilder] = None,
-                 container_dir: Optional[str] = None,
+                 key_builder: KeyBuilder | None = None,
+                 container_dir: str | None = None,
                  *,
                  enable_wal: bool = False,
-                 safe_sync: Optional[bool] = None) -> None:
+                 safe_sync: bool | None = None) -> None:
         """
         :arg identifier: a filename-compatible string identifying this
             dictionary
@@ -840,7 +840,7 @@ class PersistentDict(_PersistentDictBase[K, V]):
     def fetch(self, key: K) -> V:
         keyhash = self.key_builder(key)
 
-        def fetch_inner() -> Optional[Tuple[Any]]:
+        def fetch_inner() -> tuple[Any] | None:
             assert self.conn is not None
 
             # This is separate from fetch() so that the mutex covers the
@@ -858,7 +858,7 @@ class PersistentDict(_PersistentDictBase[K, V]):
 
         stored_key, value = pickle.loads(row[0])
         self._collision_check(key, stored_key)
-        return cast(V, value)
+        return cast("V", value)
 
     def remove(self, key: K) -> None:
         """Remove the entry associated with *key* from the dictionary."""
