@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+
 __copyright__ = "Copyright (C) 2009-2013 Andreas Kloeckner"
 
 __license__ = """
@@ -21,13 +24,14 @@ THE SOFTWARE.
 """
 
 import marshal
+from importlib.util import MAGIC_NUMBER as BYTECODE_VERSION
 from types import FunctionType, ModuleType
 
-from pytools.codegen import CodeGenerator as CodeGeneratorBase
-from pytools.codegen import Indentation, remove_common_indentation  # noqa
-
-
-from importlib.util import MAGIC_NUMBER as BYTECODE_VERSION
+from pytools.codegen import (  # noqa: F401
+    CodeGenerator as CodeGeneratorBase,
+    Indentation,
+    remove_common_indentation,
+)
 
 
 class PythonCodeGenerator(CodeGeneratorBase):
@@ -110,9 +114,9 @@ class PicklableModule:
             raise ValueError("unknown version of PicklableModule")
 
         if magic != BYTECODE_VERSION:
-            raise ValueError("cannot unpickle function binary: "
-                    "incorrect magic value (got: %r, expected: %r)"
-                    % (magic, BYTECODE_VERSION))
+            raise ValueError(
+                    "cannot unpickle function binary: incorrect magic value "
+                    f"(got: {magic!r}, expected: {BYTECODE_VERSION!r})")
 
         mod_globals = _empty_module_dict.copy()
         mod_globals.update(nondefault_globals)
@@ -135,7 +139,7 @@ class PicklableModule:
 # {{{ picklable function
 
 class PicklableFunction:
-    """Convience class wrapping a function in a :class:`PicklableModule`.
+    """Convenience class wrapping a function in a :class:`PicklableModule`.
     """
 
     def __init__(self, module, name):
