@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+
 __copyright__ = "Copyright (C) 2009-2021 Andreas Kloeckner"
 
 __license__ = """
@@ -24,7 +27,6 @@ THE SOFTWARE.
 import logging
 import sys
 from dataclasses import dataclass
-from typing import FrozenSet
 
 import pytest
 
@@ -403,7 +405,7 @@ class FakeArray:
     def __getitem__(self, idx):
         FakeArray.nopes += 1
         if idx > 10:
-            raise IndexError()
+            raise IndexError
 
 
 def test_make_obj_array_iteration():
@@ -509,7 +511,7 @@ def test_tag() -> None:
     # Need a subclass that defines the copy function in order to test.
     @tag_dataclass
     class TaggableWithCopy(Taggable):
-        tags: FrozenSet[Tag]
+        tags: frozenset[Tag]
 
         def _with_new_tags(self, tags):
             return TaggableWithCopy(tags)
@@ -869,6 +871,22 @@ def test_record():
     # }}}
 
 
+def test_permutations():
+    from math import factorial
+
+    from pytools import generate_permutations, generate_unique_permutations
+
+    perm = list(generate_permutations([1, 2, 3, 4]))
+    assert len(perm)  == factorial(4)
+    perm = list(generate_unique_permutations((1, 3, 3, 4)))
+    assert len(perm)  == 12
+
+    perms = list(generate_permutations("1234"))
+    assert len(perms)  == factorial(4)
+    perms = list(generate_unique_permutations("1334"))
+    assert len(perms)  == 12
+
+
 if __name__ == "__main__":
     if len(sys.argv) > 1:
         exec(sys.argv[1])
